// -*- coding: utf-8 -*-
// vi:si:et:sw=2:sts=4:ts=2

Components.utils.import("resource://ox/oxff.jsm");

function Site(site, access) {
  this.site = site;
  this.access = access;
  this.status = access==1?"Allowed":"Denied";
}

Site.prototype = {
  delete: function() {
    var conn = oxff.getDB();
    var q = conn.createStatement("DELETE FROM site WHERE site = :site");
    q.params.site = this.site;
    q.executeStep();
  },
  toggle: function() {
    var conn = oxff.getDB();
    if(this.access==1) this.access=0;
    else this.access = 1;
    var q = conn.createStatement("UPDATE site SET access = :access WHERE site = :site");
    q.params.site = this.site;
    q.params.access = this.access;
    q.executeStep();
    this.status = this.access==1?"Allowed":"Denied";
  }
}

var OxFFManager = {
  _sites  : [],
  _tree         : null,
  _view: {
    _rowCount: 0,
    get rowCount() {
      return this._rowCount;
    },
    getCellText: function (aRow, aColumn) {
      if (aColumn.id == "siteCol")
        return OxFFManager._sites[aRow].site;
      else if (aColumn.id == "statusCol")
        return OxFFManager._sites[aRow].status;
      return "";
    },
    isSeparator: function(aIndex) { return false; },
    isSorted: function() { return false; },
    isContainer: function(aIndex) { return false; },
    setTree: function(aTree){},
    getImageSrc: function(aRow, aColumn) {},
    getProgressMode: function(aRow, aColumn) {},
    getCellValue: function(aRow, aColumn) {},
    cycleHeader: function(column) {},
    getRowProperties: function(row,prop){},
    getColumnProperties: function(column,prop){},
    getCellProperties: function(row,column,prop){}
  },
  onLoad: function () {
    this._tree = document.getElementById("sitesTree");
    this._sites = [];

    // load permissions into a table
    var conn = oxff.getDB();
    var q = conn.createStatement("SELECT site, access FROM site");
    while(q.executeStep()) {
        var p = new Site(q.row.site, q.row.access);
        this._sites.push(p);
    }

    this._view._rowCount = this._sites.length;

    // sort and display the table
    this._tree.treeBoxObject.view = this._view;
    this.onSiteSort("site", false);

    // disable "remove all" button if there are none
    document.getElementById("removeAllSites").disabled = this._sites.length == 0;
  },
  onSiteDoubleclick: function (aEvent) {
    var selection = this._tree.view.selection;
    var rc = selection.getRangeCount();
    for (var i = 0; i < rc; ++i) {
      var min = { }; var max = { };
      selection.getRangeAt(i, min, max);
      for (var j = min.value; j <= max.value; ++j) {
        this._sites[j].toggle();
      }
    }
  },
  onSiteSelected: function () {
    var hasSelection = this._tree.view.selection.count > 0;
    var hasRows = this._tree.view.rowCount > 0;
    document.getElementById("removeSite").disabled = !hasRows || !hasSelection;
    document.getElementById("removeAllSites").disabled = !hasRows;
  },
  onSiteDeleted: function () {
    if (!this._view.rowCount)
      return;
    var removedSites = [];
    gTreeUtils.deleteSelectedItems(this._tree, this._view, this._sites, removedSites);
    for (var i = 0; i < removedSites.length; ++i) {
      var p = removedSites[i];
      p.delete();
    }
    document.getElementById("removeSite").disabled = !this._sites.length;
    document.getElementById("removeAllSites").disabled = !this._sites.length;
  },
  onAllSitesDeleted: function () {
    if (!this._view.rowCount)
      return;
    var removedSites = [];
    gTreeUtils.deleteAll(this._tree, this._view, this._sites, removedSites);
    for (var i = 0; i < removedSites.length; ++i) {
      var p = removedSites[i];
      p.delete();
    }
    document.getElementById("removeSite").disabled = true;
    document.getElementById("removeAllSites").disabled = true;
  },
  _lastSiteSortColumn: "",
  _lastSiteSortAscending: false,

  onSiteSort: function (aColumn) {
    this._lastSiteSortAscending = gTreeUtils.sort(this._tree,
                                                        this._view,
                                                        this._sites,
                                                        aColumn,
                                                        this._lastSiteSortColumn,
                                                        this._lastSiteSortAscending);
    this._lastSiteSortColumn = aColumn;
  },
};

var gTreeUtils = {
  deleteAll: function (aTree, aView, aItems, aDeletedItems) {
    for (var i = 0; i < aItems.length; ++i)
      aDeletedItems.push(aItems[i]);
    aItems.splice(0);
    var oldCount = aView.rowCount;
    aView._rowCount = 0;
    aTree.treeBoxObject.rowCountChanged(0, -oldCount);
  },

  deleteSelectedItems: function (aTree, aView, aItems, aDeletedItems) {
    var selection = aTree.view.selection;
    selection.selectEventsSuppressed = true;

    var rc = selection.getRangeCount();
    for (var i = 0; i < rc; ++i) {
      var min = { }; var max = { };
      selection.getRangeAt(i, min, max);
      for (var j = min.value; j <= max.value; ++j) {
        aDeletedItems.push(aItems[j]);
        aItems[j] = null;
      }
    }

    var nextSelection = 0;
    for (i = 0; i < aItems.length; ++i) {
      if (!aItems[i]) {
        var j = i;
        while (j < aItems.length && !aItems[j])
          ++j;
        aItems.splice(i, j - i);
        nextSelection = j < aView.rowCount ? j - 1 : j - 2;
        aView._rowCount -= j - i;
        aTree.treeBoxObject.rowCountChanged(i, i - j);
      }
    }

    if (aItems.length) {
      selection.select(nextSelection);
      aTree.treeBoxObject.ensureRowIsVisible(nextSelection);
      aTree.focus();
    }
    selection.selectEventsSuppressed = false;
  },

  sort: function (aTree, aView, aDataSet, aColumn,
                  aLastSortColumn, aLastSortAscending) {
    var ascending = (aColumn == aLastSortColumn) ? !aLastSortAscending : true;
    aDataSet.sort(function (a, b) { return a[aColumn].toLowerCase().localeCompare(b[aColumn].toLowerCase()); });
    if (!ascending)
      aDataSet.reverse();

    aTree.view.selection.select(-1);
    aTree.view.selection.select(0);
    aTree.treeBoxObject.invalidate();
    aTree.treeBoxObject.ensureRowIsVisible(0);

    return ascending;
  }
};
