// -*- coding: utf-8 -*-
// vi:si:et:sw=2:sts=2:ts=2:ft=js
/*

  OxFF - local extension for pan.do/ra
              http://pan.do/ra
              2009-2010 - GPL 3.0
 */

const Cc = Components.classes;
const Ci = Components.interfaces;

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");

Components.utils.import("resource://firefogg/uploader.jsm");
Components.utils.import("resource://firefogg/encoder.jsm");
Components.utils.import("resource://firefogg/utils.jsm");

Components.utils.import("resource://ox/oxff.jsm");

var OxFFFactory =
{
  createInstance: function (outer, iid)
  {
    if (outer != null)
      throw NS_ERROR_NO_AGGREGATION;

    return (new OxFF()).QueryInterface(iid);
  }
};

function OxFF() {
  var _this = this;
  var windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
  var nsWindow = windowMediator.getMostRecentWindow("navigator:browser");
  this._window = nsWindow.content;
  this._window.addEventListener("unload", function() { _this.logout() }, false);

  this.app = Cc["@mozilla.org/fuel/application;1"].getService(Ci.fuelIApplication);

  this._site = this._window.document.location.hostname;
  if (!this._site) {
      this._site = 'localhost';
  }
  this.access();

  if (!oxff.get('username')) {
    oxff.set('username', utils.makeRandomString(8));
    oxff.set('password', utils.makeRandomString(8));
  }
  this.authkey = oxff.get('username');
  this.password = oxff.get('password');

  this._ffmpeg2theora = ffenc.bin('ffmpeg2theora');
}

OxFF.prototype = {
  classDescription: "OxFF pan.do/ra Firefox extension",
  classID:          Components.ID("{32e2138b-2026-4cac-87c7-4f97ac4cf1c5}"),
  contractID:       "@pan.do/ra_extension;1",
  extensionID:       "OxFF@pan.do",

  _xpcom_factory : OxFFFactory,

  _xpcom_categories : [{
    category: "JavaScript global constructor",
    entry: "OxFF"
  }],

  _prefs : Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch),
  _window : null,
  _icon : null,
  _protocolCallbacks : {},

  QueryInterface: XPCOMUtils.generateQI(
    [Ci.nsIOxFF,
     Ci.nsISecurityCheckedComponent,
     Ci.nsISupportsWeakReference,
     Ci.nsIClassInfo]),

  // nsIClassInfo
  implementationLanguage: Ci.nsIProgrammingLanguage.JAVASCRIPT,
  flags: Ci.nsIClassInfo.DOM_OBJECT,

  getInterfaces: function getInterfaces(aCount) {
    var interfaces = [Ci.nsIOxFF,
                      Ci.nsISecurityCheckedComponent,
                      Ci.nsISupportsWeakReference,
                      Ci.nsIClassInfo];
    aCount.value = interfaces.length;
    return interfaces;
  },

  getHelperForLanguage: function getHelperForLanguage(aLanguage) {
    return null;
  },

  //nsISecurityCheckedComponent
  canCallMethod: function canCallMethod(iid, methodName) {
    Components.utils.reportError(methodName);
    return "AllAccess";
  },

  canCreateWrapper: function canCreateWrapper(iid) {
    return "AllAccess";
  },

  canGetProperty: function canGetProperty(iid, propertyName) {
    Components.utils.reportError(propertyName);
    return "AllAccess";
  },

  canSetProperty: function canSetProperty(iid, propertyName) {
    Components.utils.reportError(propertyName);
    return "NoAccess";
  },
  debug: function() {
    var msg = this.extensionID + ": ";
    for(var i=0;i<arguments.length;i++) {
      msg += arguments[i];
      if (i+1<arguments.length)
        msg += ', ';
    }
    this.app.console.log(msg);
  },
  base: 'http://127.0.0.1:2620/',
  _user: null,

  //nsIOxFF
  version: "bzr",

  access: function(callback) {
    var _this = this;

    var conn = oxff.getDB();
    var q = conn.createStatement("SELECT access FROM site WHERE site = :site");
    q.params.site = this._site;
    this._access = false;  
    while(q.executeStep()) {
      if (q.row.access == 1) {
        this._access = true;
      }
    }
    q.finalize();

    if (callback && !this._access) {
      var windowMediator = Cc["@mozilla.org/appshell/window-mediator;1"].getService(Ci.nsIWindowMediator);
      var nsWindow = windowMediator.getMostRecentWindow("navigator:browser");
      var box  = nsWindow.gBrowser.getNotificationBox();
      var buttons = [{ 
        label        : "Allow",
        accessKey    : "A",
        callback     : function() { _this.permitAccess();
                                    if(callback) callback.callback(true);
        }
      },
      {
        label        : "Deny",
        accessKey    : "D",
        callback     : function() { _this.denyAccess();
                                    if(callback) callback.callback(false);
        }
      }];
      box.appendNotification("Do you wnat to allow "+_this._site+" to manage archvies on your computer",
                             'oxff_permission' , null , box.PRIORITY_INFO_MEDIUM , buttons);
    } else if(callback) {
      callback.callback(this._access);
    }
    return this._access;
  },
  api: function(action, data, callback) {
    var _this = this;
    if (typeof(data) == 'function') {
        callback = data;
        data = {};
    }
    if (!this.canAccess())
      return false;

    data["key"] = this.authkey;
    data["site"] = this._site;
    data["user"] = this._user;
    
    var base = this.base; //later base can not be a public property
    var url = base + action;

    utils.request({
      url: url,
      data: data,
      load: function(e) {
          //links should have base prefixed or whatever proxy is used to access them, i.e. some api call
          //var data = JSON.parse(e.target.responseText);
          //callback(data);
          callback(e.target.responseText);
      },
      error: function(e) {
        _this.startDaemon();
        utils.setTimeout(function() { _this.api(action, data, callback); }, 1000);
      }
    });
    return true;
  },
  chunk_upload: function(url, data, file, callback, progress) {
    if (!this._chunk_upload) {
      var _this = this;
      this._chunk_upload = new FirefoggUploader(this, file, url, data);
      this._chunk_upload.onProgress(function(upload_progress) {
        var info = {'status': 'uploading', 'progress': upload_progress};
        progress(JSON.stringify(info));
      });
      this._chunk_upload.ready = true;
      this._chunk_upload.start();
      this._chunk_upload.done(function() {
        _this._chunk_upload = null;
        var info = {'status': 'done', 'progress': 1};
        callback(JSON.stringify(info));
      });
      return true;
    }
    return false;
  },
  upload: function(options, callback, progress) {
    var _this = this;

    if (!this.canAccess())
      return false;

    var options = JSON.parse(options);
    if (options.data)
      var formData = options.data;
    else
      var formData = {};

    var url = options.url;

    if (callback)
      callback = callback.callback;
    if (progress)
      progress = progress.callback;

    if (options.action == 'frames') {
      var timer = utils.setInterval(function() {
        _this.api('extract', {oshash: options.oshash, media: 'frames'}, function(result) {
          var data = JSON.parse(result);
          if (data.status == 'extracting') {
            if (progress && data.progress >= 0) {
              var info = {'status': data.status, 'progress': data.progress};
              progress(JSON.stringify(info));
            }
          } else {
            timer.cancel();
            if (data.status == 'available') {
              formData.frame = new Array();
              for(i in data.frames) {
                var f  = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
                f.initWithPath(data.frames[i]);
                formData.frame.push(f);
              }
              utils.request({
                url: url,
                data: formData,
                load: function(e) {
                    if (callback)
                      callback(e.target.responseText);
                },
                error: function(e) {
                  _this.debug('uploading frames failed');
                }
              });
            } else if (data.status == 'failed') {
              _this.debug('uploading frames failed');
            }
          }
        });
      }, 2000);
    } else if (options.action == 'file') { //FIXME: should this use Firefogg style upload for lager files?
      var conn = oxff.getDB();
      var q = conn.createStatement("SELECT path FROM file WHERE oshash = :oshash LIMIT 1");
      q.params.oshash = options.oshash;
      if (q.executeStep())
        var path = q.row.path;
      q.finalize();
      if (path) {
        formData.file = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile)
        formData.file.initWithPath(path);
        if (formData.file.exists()) { //FIXME: should check if more than one file with oshash is known
          utils.request({
            url: url,
            data: formData,
            load: function(e) {
                if (callback)
                  callback(e.target.responseText);
            },
            error: function(e) {
              _this.debug('uploading file failed');
            }
          });
          return true;
        }
      }
      return false;
    } else if (options.action == 'video') {
      var timer = utils.setInterval(function() {
        _this.api('extract', {oshash: options.oshash, media: options.profile}, function(result) {
          var data = JSON.parse(result);
          if (data.status == 'extracting') {
            if (progress && data.progress >= 0) {
              var info = {'status': data.status, 'progress': data.progress};
              progress(JSON.stringify(info));
            }
          } else {
            timer.cancel();
            if (data.status == 'available') {
              _this.chunk_upload(url, formData, data.video, callback, progress);
            } else if (data.status == 'failed') {
              _this.debug('uploading video failed');
            }
          }
        });
      }, 2000);

    }
    return true;
  },
  files: function(volume, callback) {
    return this.api('files', {'volume': volume}, callback.callback);
  },
  get: function(oshash, media, callback) {
    return this.api('get', {'oshash': oshash, 'media': media}, callback.callback);
  },
  login: function(user) {
    if (this._user==null) {
      this._user = user;
      return true;
    }
    return false;
  },
  logout: function(user) {
    var _this = this;
    this.api('stop', function() {
      _this._user = null;
      _this._daemon = null;
    });
    if (this._chunk_upload) {
      this._chunk_upload.cancel();
      this._chunk_upload = null;
    }
    return true;
  },
  update: function(callback) {
    this.api('update', callback.callback);
    return true;
  },
  setLocation: function(name) {
    if (!this._access) {
        return false;
    }
    const nsIFilePicker = Ci.nsIFilePicker;
    var fp = Cc["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
    fp.init(this._window, "Set Locatin of "+name+" for " + this._site, nsIFilePicker.modeGetFolder);
    fp.appendFilters(nsIFilePicker.filterAll);
    var rv = fp.show();
    if (rv == nsIFilePicker.returnOK || rv == nsIFilePicker.returnReplace) {
        var new_location = fp.file.path;
        this.api('set_location', {'name': name, 'path': new_location}, function(result) {});
        return true;
    }
    return false;
  },
  removeVolume: function(name) {
    return this.api('remove_volume', {'name': name});
  },
  renameVolume: function(name, new_name) {
    return this.api('rename_volume', {'name': name, 'new_name': new_name});
  },
  volumes: function(callback) {
    return this.api('volumes', callback.callback);
  },
  //private functions
  permitAccess: function() {
    this._access = true;
    oxff.access(this._site, this._access);
  },
  denyAccess: function() {
    this._access = false;
    oxff.access(this._site, this._access);
  },
  canAccess: function() {
      if (!this._access) {
        this.debug('permission deinied');
        return false;
    }
    if (!this._user) {
        this.debug('login first');
        return false;
    }
    return true;
  },
  startDaemon: function() {
    var _this = this;
    //if daemon is already running to not start it again
    if (this._daemon && this._daemon.isRunning) {
      this.debug('daemon is still starting up, not starting again');
      return false;
    }
    var osString = Cc["@mozilla.org/xre/app-info;1"].getService(Ci.nsIXULRuntime).OS;  
    var daemon = "oxd.py";
    if (osString == "WINNT")
      daemon = 'oxd.exe';

    function runDaemon(file) {
      try {
          file.permissions = 0755;
      } catch (e) {}
      _this._daemon = Cc["@mozilla.org/process/util;1"].createInstance(Ci.nsIProcess);
      _this._daemon.init(file);
      var args = [oxff.getDBFile().path, _this._ffmpeg2theora];
      _this._daemon.runw(false, args, args.length);
    }
    var file = __LOCATION__.parent.parent.QueryInterface(Ci.nsILocalFile);
    file.appendRelativePath('bin');
    file.appendRelativePath(daemon);
    runDaemon(file);
  },
}

/**
* XPCOMUtils.generateNSGetFactory was introduced in Mozilla 2 (Firefox 4).
* XPCOMUtils.generateNSGetModule is for Mozilla 1.9.2 (Firefox 3.6).
*/
if (XPCOMUtils.generateNSGetFactory)
  var NSGetFactory = XPCOMUtils.generateNSGetFactory([OxFF]);
else
  var NSGetModule = XPCOMUtils.generateNSGetModule([OxFF]);

